#include "driver/i2c.h"
#define I2C_MASTER_SCL_IO           22      	/*!< GPIO number used for I2C master clock */
#define I2C_MASTER_SDA_IO           21      	/*!< GPIO number used for I2C master data  */
#define I2C_MASTER_NUM              0           /*!< I2C master i2c port number, the number of i2c peripheral interfaces available will depend on the chip */
#define I2C_MASTER_FREQ_HZ          400000      /*!< I2C master clock frequency */
#define I2C_MASTER_TX_BUF_DISABLE   0           /*!< I2C master doesn't need buffer */
#define I2C_MASTER_RX_BUF_DISABLE   0           /*!< I2C master doesn't need buffer */
#define I2C_MASTER_TIMEOUT_MS       1000

#define AQM0802A_ADDR		        0x3e        

// commands

#define LCD_CLEARDISPLAY 			0x01
#define LCD_RETURNHOME 				0x02
#define LCD_ENTRYMODESET 			0x04
#define LCD_DISPLAYCONTROL 			0x08
#define LCD_CURSORSHIFT 			0x10
#define LCD_FUNCTIONSET 			0x20
#define LCD_SETCGRAMADDR 			0x40
#define LCD_SETDDRAMADDR 			0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 				0x00
#define LCD_ENTRYLEFT 				0x02
#define LCD_ENTRYSHIFTINCREMENT 	0x01
#define LCD_ENTRYSHIFTDECREMENT 	0x00

// flags for display on/off control
#define LCD_DISPLAYON 				0x04
#define LCD_DISPLAYOFF 				0x00
#define LCD_CURSORON 				0x02
#define LCD_CURSOROFF 				0x00
#define LCD_BLINKON 				0x01
#define LCD_BLINKOFF 				0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 			0x08
#define LCD_CURSORMOVE 				0x00
#define LCD_MOVERIGHT 				0x04
#define LCD_MOVELEFT 				0x00

void AQM0802A_init(void);
void AQM0802A_print(char* buf);
//------------------- high level commands, for the user! ------------------
void AQM0802A_clear(void);
void AQM0802A_home(void);
void AQM0802A_setCursor(uint8_t col, uint8_t row);
void AQM0802A_noDisplay(void);
void AQM0802A_display(void);
void AQM0802A_noCursor(void);
void AQM0802A_cursor(void);
void AQM0802A_noBlink(void);
void AQM0802A_blink(void);
void AQM0802A_scrollDisplayLeft(void);
void AQM0802A_scrollDisplayRight(void);
void AQM0802A_leftToRight(void);
void AQM0802A_rightToLeft(void);
void AQM0802A_autoscroll(void);
void AQM0802A_noAutoscroll(void);
void AQM0802A_createChar(uint8_t location, uint8_t charmap[]);
void AQM0802A_setContrast(uint8_t contrast);
//-------------------  mid level commands, for sending data/cmds ------------------
void AQM0802A_command(uint8_t value);
void AQM0802A_data(uint8_t value);
esp_err_t AQM0802A_write_byte(uint8_t data, uint8_t reg_addr);
esp_err_t i2c_master_init(void);

uint8_t _displaycontrol;
uint8_t _displaymode;

